'''
Created on Dec 22, 2011

@author: sean
'''
import unittest


class Test(unittest.TestCase):


    def test_wth(self):
        pass


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.test_wth']
    unittest.main()